package clone

import api "github.com/docker/compose-on-kubernetes/api/compose/clone"

// MapOfStringToSliceOfString deep copy a map[string][]string
// Deprecated: Use github.com/docker/compose-on-kubernetes/api/compose/clone.MapOfStringToSliceOfString instead
var MapOfStringToSliceOfString = api.MapOfStringToSliceOfString

// MapOfStringToInt deep copy a map[string]int
// Deprecated: Use github.com/docker/compose-on-kubernetes/api/compose/clone.MapOfStringToInt instead
var MapOfStringToInt = api.MapOfStringToInt
