package v1beta2

import api "github.com/docker/compose-on-kubernetes/api/compose/v1beta2"

// ComposeFile is the content of a stack's compose file if any
// Deprecated: Use github.com/docker/compose-on-kubernetes/api/compose/v1beta2.ComposeFile instead
type ComposeFile = api.ComposeFile
