package v1beta2

import api "github.com/docker/compose-on-kubernetes/api/compose/v1beta2"

// Owner describes the user who created the stack
// Deprecated: Use github.com/docker/compose-on-kubernetes/api/compose/v1beta2.Owner instead
type Owner = api.Owner
