package v1beta2

import api "github.com/docker/compose-on-kubernetes/api/compose/v1beta2"

// Scale contains the current/desired replica count for services in a stack.
// Deprecated: Use github.com/docker/compose-on-kubernetes/api/compose/v1beta2.Scale instead
type Scale = api.Scale
