package v1beta1

import api "github.com/docker/compose-on-kubernetes/api/compose/v1beta1"

// MaxComposeVersion is the most recent version of compose file Schema supported in v1beta1
// Deprecated: Use github.com/docker/compose-on-kubernetes/api/compose/v1beta1.MaxComposeVersion instead
const MaxComposeVersion = api.MaxComposeVersion
