package impersonation

import api "github.com/docker/compose-on-kubernetes/api/compose/impersonation"

// Config contains the data required to impersonate a user.
// Deprecated: Use github.com/docker/compose-on-kubernetes/api/compose/impersonation.Config instead
type Config = api.Config
