package v1beta2

import api "github.com/docker/compose-on-kubernetes/api/client/informers/compose/v1beta2"

// StackInformer provides access to a shared informer and lister for
// Stacks.
// Deprecated: Use github.com/docker/compose-on-kubernetes/api/client/informers/compose/v1beta2.StackInformer instead
type StackInformer = api.StackInformer
