package clone

import api "github.com/docker/compose-on-kubernetes/api/compose/clone"

// SliceOfString deep copy a slice of strings
// Deprecated: Use github.com/docker/compose-on-kubernetes/api/compose/clone.SliceOfString instead
var SliceOfString = api.SliceOfString
