package v1beta2

import api "github.com/docker/compose-on-kubernetes/api/client/listers/compose/v1beta2"

// StackListerExpansion allows custom methods to be added to
// StackLister.
// Deprecated: Use github.com/docker/compose-on-kubernetes/api/client/listers/compose/v1beta2.StackListerExpansion instead
type StackListerExpansion = api.StackListerExpansion

// StackNamespaceListerExpansion allows custom methods to be added to
// StackNamespaceLister.
// Deprecated: Use github.com/docker/compose-on-kubernetes/api/client/listers/compose/v1beta2.StackNamespaceListerExpansion instead
type StackNamespaceListerExpansion = api.StackNamespaceListerExpansion
