// +build !ignore_autogenerated

// Code generated by conversion-gen. DO NOT EDIT.

package v1alpha3

import (
	time "time"
	unsafe "unsafe"

	v1beta2 "github.com/docker/compose-on-kubernetes/api/compose/v1beta2"
	conversion "k8s.io/apimachinery/pkg/conversion"
	runtime "k8s.io/apimachinery/pkg/runtime"
)

func init() {
	localSchemeBuilder.Register(RegisterConversions)
}

// RegisterConversions adds conversion functions to the given scheme.
// Public to allow building arbitrary schemes.
func RegisterConversions(s *runtime.Scheme) error {
	if err := s.AddGeneratedConversionFunc((*ComposeFile)(nil), (*v1beta2.ComposeFile)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_ComposeFile_To_v1beta2_ComposeFile(a.(*ComposeFile), b.(*v1beta2.ComposeFile), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.ComposeFile)(nil), (*ComposeFile)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_ComposeFile_To_v1alpha3_ComposeFile(a.(*v1beta2.ComposeFile), b.(*ComposeFile), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*ConfigObjConfig)(nil), (*v1beta2.ConfigObjConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_ConfigObjConfig_To_v1beta2_ConfigObjConfig(a.(*ConfigObjConfig), b.(*v1beta2.ConfigObjConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.ConfigObjConfig)(nil), (*ConfigObjConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_ConfigObjConfig_To_v1alpha3_ConfigObjConfig(a.(*v1beta2.ConfigObjConfig), b.(*ConfigObjConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*Constraint)(nil), (*v1beta2.Constraint)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_Constraint_To_v1beta2_Constraint(a.(*Constraint), b.(*v1beta2.Constraint), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.Constraint)(nil), (*Constraint)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_Constraint_To_v1alpha3_Constraint(a.(*v1beta2.Constraint), b.(*Constraint), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*Constraints)(nil), (*v1beta2.Constraints)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_Constraints_To_v1beta2_Constraints(a.(*Constraints), b.(*v1beta2.Constraints), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.Constraints)(nil), (*Constraints)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_Constraints_To_v1alpha3_Constraints(a.(*v1beta2.Constraints), b.(*Constraints), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*DeployConfig)(nil), (*v1beta2.DeployConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_DeployConfig_To_v1beta2_DeployConfig(a.(*DeployConfig), b.(*v1beta2.DeployConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.DeployConfig)(nil), (*DeployConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_DeployConfig_To_v1alpha3_DeployConfig(a.(*v1beta2.DeployConfig), b.(*DeployConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*External)(nil), (*v1beta2.External)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_External_To_v1beta2_External(a.(*External), b.(*v1beta2.External), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.External)(nil), (*External)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_External_To_v1alpha3_External(a.(*v1beta2.External), b.(*External), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*FileObjectConfig)(nil), (*v1beta2.FileObjectConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_FileObjectConfig_To_v1beta2_FileObjectConfig(a.(*FileObjectConfig), b.(*v1beta2.FileObjectConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.FileObjectConfig)(nil), (*FileObjectConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_FileObjectConfig_To_v1alpha3_FileObjectConfig(a.(*v1beta2.FileObjectConfig), b.(*FileObjectConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*FileReferenceConfig)(nil), (*v1beta2.FileReferenceConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_FileReferenceConfig_To_v1beta2_FileReferenceConfig(a.(*FileReferenceConfig), b.(*v1beta2.FileReferenceConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.FileReferenceConfig)(nil), (*FileReferenceConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_FileReferenceConfig_To_v1alpha3_FileReferenceConfig(a.(*v1beta2.FileReferenceConfig), b.(*FileReferenceConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*HealthCheckConfig)(nil), (*v1beta2.HealthCheckConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_HealthCheckConfig_To_v1beta2_HealthCheckConfig(a.(*HealthCheckConfig), b.(*v1beta2.HealthCheckConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.HealthCheckConfig)(nil), (*HealthCheckConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_HealthCheckConfig_To_v1alpha3_HealthCheckConfig(a.(*v1beta2.HealthCheckConfig), b.(*HealthCheckConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*Owner)(nil), (*v1beta2.Owner)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_Owner_To_v1beta2_Owner(a.(*Owner), b.(*v1beta2.Owner), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.Owner)(nil), (*Owner)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_Owner_To_v1alpha3_Owner(a.(*v1beta2.Owner), b.(*Owner), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*Placement)(nil), (*v1beta2.Placement)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_Placement_To_v1beta2_Placement(a.(*Placement), b.(*v1beta2.Placement), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.Placement)(nil), (*Placement)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_Placement_To_v1alpha3_Placement(a.(*v1beta2.Placement), b.(*Placement), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*Resource)(nil), (*v1beta2.Resource)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_Resource_To_v1beta2_Resource(a.(*Resource), b.(*v1beta2.Resource), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.Resource)(nil), (*Resource)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_Resource_To_v1alpha3_Resource(a.(*v1beta2.Resource), b.(*Resource), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*Resources)(nil), (*v1beta2.Resources)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_Resources_To_v1beta2_Resources(a.(*Resources), b.(*v1beta2.Resources), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.Resources)(nil), (*Resources)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_Resources_To_v1alpha3_Resources(a.(*v1beta2.Resources), b.(*Resources), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*RestartPolicy)(nil), (*v1beta2.RestartPolicy)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_RestartPolicy_To_v1beta2_RestartPolicy(a.(*RestartPolicy), b.(*v1beta2.RestartPolicy), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.RestartPolicy)(nil), (*RestartPolicy)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_RestartPolicy_To_v1alpha3_RestartPolicy(a.(*v1beta2.RestartPolicy), b.(*RestartPolicy), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*Scale)(nil), (*v1beta2.Scale)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_Scale_To_v1beta2_Scale(a.(*Scale), b.(*v1beta2.Scale), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.Scale)(nil), (*Scale)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_Scale_To_v1alpha3_Scale(a.(*v1beta2.Scale), b.(*Scale), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*SecretConfig)(nil), (*v1beta2.SecretConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_SecretConfig_To_v1beta2_SecretConfig(a.(*SecretConfig), b.(*v1beta2.SecretConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.SecretConfig)(nil), (*SecretConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_SecretConfig_To_v1alpha3_SecretConfig(a.(*v1beta2.SecretConfig), b.(*SecretConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*ServiceConfig)(nil), (*v1beta2.ServiceConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_ServiceConfig_To_v1beta2_ServiceConfig(a.(*ServiceConfig), b.(*v1beta2.ServiceConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.ServiceConfig)(nil), (*ServiceConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_ServiceConfig_To_v1alpha3_ServiceConfig(a.(*v1beta2.ServiceConfig), b.(*ServiceConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*ServiceConfigObjConfig)(nil), (*v1beta2.ServiceConfigObjConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_ServiceConfigObjConfig_To_v1beta2_ServiceConfigObjConfig(a.(*ServiceConfigObjConfig), b.(*v1beta2.ServiceConfigObjConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.ServiceConfigObjConfig)(nil), (*ServiceConfigObjConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_ServiceConfigObjConfig_To_v1alpha3_ServiceConfigObjConfig(a.(*v1beta2.ServiceConfigObjConfig), b.(*ServiceConfigObjConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*ServicePortConfig)(nil), (*v1beta2.ServicePortConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_ServicePortConfig_To_v1beta2_ServicePortConfig(a.(*ServicePortConfig), b.(*v1beta2.ServicePortConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.ServicePortConfig)(nil), (*ServicePortConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_ServicePortConfig_To_v1alpha3_ServicePortConfig(a.(*v1beta2.ServicePortConfig), b.(*ServicePortConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*ServiceSecretConfig)(nil), (*v1beta2.ServiceSecretConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_ServiceSecretConfig_To_v1beta2_ServiceSecretConfig(a.(*ServiceSecretConfig), b.(*v1beta2.ServiceSecretConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.ServiceSecretConfig)(nil), (*ServiceSecretConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_ServiceSecretConfig_To_v1alpha3_ServiceSecretConfig(a.(*v1beta2.ServiceSecretConfig), b.(*ServiceSecretConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*ServiceVolumeConfig)(nil), (*v1beta2.ServiceVolumeConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_ServiceVolumeConfig_To_v1beta2_ServiceVolumeConfig(a.(*ServiceVolumeConfig), b.(*v1beta2.ServiceVolumeConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.ServiceVolumeConfig)(nil), (*ServiceVolumeConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_ServiceVolumeConfig_To_v1alpha3_ServiceVolumeConfig(a.(*v1beta2.ServiceVolumeConfig), b.(*ServiceVolumeConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*Stack)(nil), (*v1beta2.Stack)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_Stack_To_v1beta2_Stack(a.(*Stack), b.(*v1beta2.Stack), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.Stack)(nil), (*Stack)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_Stack_To_v1alpha3_Stack(a.(*v1beta2.Stack), b.(*Stack), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*StackList)(nil), (*v1beta2.StackList)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_StackList_To_v1beta2_StackList(a.(*StackList), b.(*v1beta2.StackList), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.StackList)(nil), (*StackList)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_StackList_To_v1alpha3_StackList(a.(*v1beta2.StackList), b.(*StackList), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*StackSpec)(nil), (*v1beta2.StackSpec)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_StackSpec_To_v1beta2_StackSpec(a.(*StackSpec), b.(*v1beta2.StackSpec), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.StackSpec)(nil), (*StackSpec)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_StackSpec_To_v1alpha3_StackSpec(a.(*v1beta2.StackSpec), b.(*StackSpec), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*StackStatus)(nil), (*v1beta2.StackStatus)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_StackStatus_To_v1beta2_StackStatus(a.(*StackStatus), b.(*v1beta2.StackStatus), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.StackStatus)(nil), (*StackStatus)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_StackStatus_To_v1alpha3_StackStatus(a.(*v1beta2.StackStatus), b.(*StackStatus), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*UpdateConfig)(nil), (*v1beta2.UpdateConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1alpha3_UpdateConfig_To_v1beta2_UpdateConfig(a.(*UpdateConfig), b.(*v1beta2.UpdateConfig), scope)
	}); err != nil {
		return err
	}
	if err := s.AddGeneratedConversionFunc((*v1beta2.UpdateConfig)(nil), (*UpdateConfig)(nil), func(a, b interface{}, scope conversion.Scope) error {
		return Convert_v1beta2_UpdateConfig_To_v1alpha3_UpdateConfig(a.(*v1beta2.UpdateConfig), b.(*UpdateConfig), scope)
	}); err != nil {
		return err
	}
	return nil
}

func autoConvert_v1alpha3_ComposeFile_To_v1beta2_ComposeFile(in *ComposeFile, out *v1beta2.ComposeFile, s conversion.Scope) error {
	out.ObjectMeta = in.ObjectMeta
	out.ComposeFile = in.ComposeFile
	return nil
}

// Convert_v1alpha3_ComposeFile_To_v1beta2_ComposeFile is an autogenerated conversion function.
func Convert_v1alpha3_ComposeFile_To_v1beta2_ComposeFile(in *ComposeFile, out *v1beta2.ComposeFile, s conversion.Scope) error {
	return autoConvert_v1alpha3_ComposeFile_To_v1beta2_ComposeFile(in, out, s)
}

func autoConvert_v1beta2_ComposeFile_To_v1alpha3_ComposeFile(in *v1beta2.ComposeFile, out *ComposeFile, s conversion.Scope) error {
	out.ObjectMeta = in.ObjectMeta
	out.ComposeFile = in.ComposeFile
	return nil
}

// Convert_v1beta2_ComposeFile_To_v1alpha3_ComposeFile is an autogenerated conversion function.
func Convert_v1beta2_ComposeFile_To_v1alpha3_ComposeFile(in *v1beta2.ComposeFile, out *ComposeFile, s conversion.Scope) error {
	return autoConvert_v1beta2_ComposeFile_To_v1alpha3_ComposeFile(in, out, s)
}

func autoConvert_v1alpha3_ConfigObjConfig_To_v1beta2_ConfigObjConfig(in *ConfigObjConfig, out *v1beta2.ConfigObjConfig, s conversion.Scope) error {
	out.Name = in.Name
	out.File = in.File
	if err := Convert_v1alpha3_External_To_v1beta2_External(&in.External, &out.External, s); err != nil {
		return err
	}
	out.Labels = *(*map[string]string)(unsafe.Pointer(&in.Labels))
	return nil
}

// Convert_v1alpha3_ConfigObjConfig_To_v1beta2_ConfigObjConfig is an autogenerated conversion function.
func Convert_v1alpha3_ConfigObjConfig_To_v1beta2_ConfigObjConfig(in *ConfigObjConfig, out *v1beta2.ConfigObjConfig, s conversion.Scope) error {
	return autoConvert_v1alpha3_ConfigObjConfig_To_v1beta2_ConfigObjConfig(in, out, s)
}

func autoConvert_v1beta2_ConfigObjConfig_To_v1alpha3_ConfigObjConfig(in *v1beta2.ConfigObjConfig, out *ConfigObjConfig, s conversion.Scope) error {
	out.Name = in.Name
	out.File = in.File
	if err := Convert_v1beta2_External_To_v1alpha3_External(&in.External, &out.External, s); err != nil {
		return err
	}
	out.Labels = *(*map[string]string)(unsafe.Pointer(&in.Labels))
	return nil
}

// Convert_v1beta2_ConfigObjConfig_To_v1alpha3_ConfigObjConfig is an autogenerated conversion function.
func Convert_v1beta2_ConfigObjConfig_To_v1alpha3_ConfigObjConfig(in *v1beta2.ConfigObjConfig, out *ConfigObjConfig, s conversion.Scope) error {
	return autoConvert_v1beta2_ConfigObjConfig_To_v1alpha3_ConfigObjConfig(in, out, s)
}

func autoConvert_v1alpha3_Constraint_To_v1beta2_Constraint(in *Constraint, out *v1beta2.Constraint, s conversion.Scope) error {
	out.Value = in.Value
	out.Operator = in.Operator
	return nil
}

// Convert_v1alpha3_Constraint_To_v1beta2_Constraint is an autogenerated conversion function.
func Convert_v1alpha3_Constraint_To_v1beta2_Constraint(in *Constraint, out *v1beta2.Constraint, s conversion.Scope) error {
	return autoConvert_v1alpha3_Constraint_To_v1beta2_Constraint(in, out, s)
}

func autoConvert_v1beta2_Constraint_To_v1alpha3_Constraint(in *v1beta2.Constraint, out *Constraint, s conversion.Scope) error {
	out.Value = in.Value
	out.Operator = in.Operator
	return nil
}

// Convert_v1beta2_Constraint_To_v1alpha3_Constraint is an autogenerated conversion function.
func Convert_v1beta2_Constraint_To_v1alpha3_Constraint(in *v1beta2.Constraint, out *Constraint, s conversion.Scope) error {
	return autoConvert_v1beta2_Constraint_To_v1alpha3_Constraint(in, out, s)
}

func autoConvert_v1alpha3_Constraints_To_v1beta2_Constraints(in *Constraints, out *v1beta2.Constraints, s conversion.Scope) error {
	out.OperatingSystem = (*v1beta2.Constraint)(unsafe.Pointer(in.OperatingSystem))
	out.Architecture = (*v1beta2.Constraint)(unsafe.Pointer(in.Architecture))
	out.Hostname = (*v1beta2.Constraint)(unsafe.Pointer(in.Hostname))
	out.MatchLabels = *(*map[string]v1beta2.Constraint)(unsafe.Pointer(&in.MatchLabels))
	return nil
}

// Convert_v1alpha3_Constraints_To_v1beta2_Constraints is an autogenerated conversion function.
func Convert_v1alpha3_Constraints_To_v1beta2_Constraints(in *Constraints, out *v1beta2.Constraints, s conversion.Scope) error {
	return autoConvert_v1alpha3_Constraints_To_v1beta2_Constraints(in, out, s)
}

func autoConvert_v1beta2_Constraints_To_v1alpha3_Constraints(in *v1beta2.Constraints, out *Constraints, s conversion.Scope) error {
	out.OperatingSystem = (*Constraint)(unsafe.Pointer(in.OperatingSystem))
	out.Architecture = (*Constraint)(unsafe.Pointer(in.Architecture))
	out.Hostname = (*Constraint)(unsafe.Pointer(in.Hostname))
	out.MatchLabels = *(*map[string]Constraint)(unsafe.Pointer(&in.MatchLabels))
	return nil
}

// Convert_v1beta2_Constraints_To_v1alpha3_Constraints is an autogenerated conversion function.
func Convert_v1beta2_Constraints_To_v1alpha3_Constraints(in *v1beta2.Constraints, out *Constraints, s conversion.Scope) error {
	return autoConvert_v1beta2_Constraints_To_v1alpha3_Constraints(in, out, s)
}

func autoConvert_v1alpha3_DeployConfig_To_v1beta2_DeployConfig(in *DeployConfig, out *v1beta2.DeployConfig, s conversion.Scope) error {
	out.Mode = in.Mode
	out.Replicas = (*uint64)(unsafe.Pointer(in.Replicas))
	out.Labels = *(*map[string]string)(unsafe.Pointer(&in.Labels))
	out.UpdateConfig = (*v1beta2.UpdateConfig)(unsafe.Pointer(in.UpdateConfig))
	if err := Convert_v1alpha3_Resources_To_v1beta2_Resources(&in.Resources, &out.Resources, s); err != nil {
		return err
	}
	out.RestartPolicy = (*v1beta2.RestartPolicy)(unsafe.Pointer(in.RestartPolicy))
	if err := Convert_v1alpha3_Placement_To_v1beta2_Placement(&in.Placement, &out.Placement, s); err != nil {
		return err
	}
	return nil
}

// Convert_v1alpha3_DeployConfig_To_v1beta2_DeployConfig is an autogenerated conversion function.
func Convert_v1alpha3_DeployConfig_To_v1beta2_DeployConfig(in *DeployConfig, out *v1beta2.DeployConfig, s conversion.Scope) error {
	return autoConvert_v1alpha3_DeployConfig_To_v1beta2_DeployConfig(in, out, s)
}

func autoConvert_v1beta2_DeployConfig_To_v1alpha3_DeployConfig(in *v1beta2.DeployConfig, out *DeployConfig, s conversion.Scope) error {
	out.Mode = in.Mode
	out.Replicas = (*uint64)(unsafe.Pointer(in.Replicas))
	out.Labels = *(*map[string]string)(unsafe.Pointer(&in.Labels))
	out.UpdateConfig = (*UpdateConfig)(unsafe.Pointer(in.UpdateConfig))
	if err := Convert_v1beta2_Resources_To_v1alpha3_Resources(&in.Resources, &out.Resources, s); err != nil {
		return err
	}
	out.RestartPolicy = (*RestartPolicy)(unsafe.Pointer(in.RestartPolicy))
	if err := Convert_v1beta2_Placement_To_v1alpha3_Placement(&in.Placement, &out.Placement, s); err != nil {
		return err
	}
	return nil
}

// Convert_v1beta2_DeployConfig_To_v1alpha3_DeployConfig is an autogenerated conversion function.
func Convert_v1beta2_DeployConfig_To_v1alpha3_DeployConfig(in *v1beta2.DeployConfig, out *DeployConfig, s conversion.Scope) error {
	return autoConvert_v1beta2_DeployConfig_To_v1alpha3_DeployConfig(in, out, s)
}

func autoConvert_v1alpha3_External_To_v1beta2_External(in *External, out *v1beta2.External, s conversion.Scope) error {
	out.Name = in.Name
	out.External = in.External
	return nil
}

// Convert_v1alpha3_External_To_v1beta2_External is an autogenerated conversion function.
func Convert_v1alpha3_External_To_v1beta2_External(in *External, out *v1beta2.External, s conversion.Scope) error {
	return autoConvert_v1alpha3_External_To_v1beta2_External(in, out, s)
}

func autoConvert_v1beta2_External_To_v1alpha3_External(in *v1beta2.External, out *External, s conversion.Scope) error {
	out.Name = in.Name
	out.External = in.External
	return nil
}

// Convert_v1beta2_External_To_v1alpha3_External is an autogenerated conversion function.
func Convert_v1beta2_External_To_v1alpha3_External(in *v1beta2.External, out *External, s conversion.Scope) error {
	return autoConvert_v1beta2_External_To_v1alpha3_External(in, out, s)
}

func autoConvert_v1alpha3_FileObjectConfig_To_v1beta2_FileObjectConfig(in *FileObjectConfig, out *v1beta2.FileObjectConfig, s conversion.Scope) error {
	out.Name = in.Name
	out.File = in.File
	if err := Convert_v1alpha3_External_To_v1beta2_External(&in.External, &out.External, s); err != nil {
		return err
	}
	out.Labels = *(*map[string]string)(unsafe.Pointer(&in.Labels))
	return nil
}

// Convert_v1alpha3_FileObjectConfig_To_v1beta2_FileObjectConfig is an autogenerated conversion function.
func Convert_v1alpha3_FileObjectConfig_To_v1beta2_FileObjectConfig(in *FileObjectConfig, out *v1beta2.FileObjectConfig, s conversion.Scope) error {
	return autoConvert_v1alpha3_FileObjectConfig_To_v1beta2_FileObjectConfig(in, out, s)
}

func autoConvert_v1beta2_FileObjectConfig_To_v1alpha3_FileObjectConfig(in *v1beta2.FileObjectConfig, out *FileObjectConfig, s conversion.Scope) error {
	out.Name = in.Name
	out.File = in.File
	if err := Convert_v1beta2_External_To_v1alpha3_External(&in.External, &out.External, s); err != nil {
		return err
	}
	out.Labels = *(*map[string]string)(unsafe.Pointer(&in.Labels))
	return nil
}

// Convert_v1beta2_FileObjectConfig_To_v1alpha3_FileObjectConfig is an autogenerated conversion function.
func Convert_v1beta2_FileObjectConfig_To_v1alpha3_FileObjectConfig(in *v1beta2.FileObjectConfig, out *FileObjectConfig, s conversion.Scope) error {
	return autoConvert_v1beta2_FileObjectConfig_To_v1alpha3_FileObjectConfig(in, out, s)
}

func autoConvert_v1alpha3_FileReferenceConfig_To_v1beta2_FileReferenceConfig(in *FileReferenceConfig, out *v1beta2.FileReferenceConfig, s conversion.Scope) error {
	out.Source = in.Source
	out.Target = in.Target
	out.UID = in.UID
	out.GID = in.GID
	out.Mode = (*uint32)(unsafe.Pointer(in.Mode))
	return nil
}

// Convert_v1alpha3_FileReferenceConfig_To_v1beta2_FileReferenceConfig is an autogenerated conversion function.
func Convert_v1alpha3_FileReferenceConfig_To_v1beta2_FileReferenceConfig(in *FileReferenceConfig, out *v1beta2.FileReferenceConfig, s conversion.Scope) error {
	return autoConvert_v1alpha3_FileReferenceConfig_To_v1beta2_FileReferenceConfig(in, out, s)
}

func autoConvert_v1beta2_FileReferenceConfig_To_v1alpha3_FileReferenceConfig(in *v1beta2.FileReferenceConfig, out *FileReferenceConfig, s conversion.Scope) error {
	out.Source = in.Source
	out.Target = in.Target
	out.UID = in.UID
	out.GID = in.GID
	out.Mode = (*uint32)(unsafe.Pointer(in.Mode))
	return nil
}

// Convert_v1beta2_FileReferenceConfig_To_v1alpha3_FileReferenceConfig is an autogenerated conversion function.
func Convert_v1beta2_FileReferenceConfig_To_v1alpha3_FileReferenceConfig(in *v1beta2.FileReferenceConfig, out *FileReferenceConfig, s conversion.Scope) error {
	return autoConvert_v1beta2_FileReferenceConfig_To_v1alpha3_FileReferenceConfig(in, out, s)
}

func autoConvert_v1alpha3_HealthCheckConfig_To_v1beta2_HealthCheckConfig(in *HealthCheckConfig, out *v1beta2.HealthCheckConfig, s conversion.Scope) error {
	out.Test = *(*[]string)(unsafe.Pointer(&in.Test))
	out.Timeout = (*time.Duration)(unsafe.Pointer(in.Timeout))
	out.Interval = (*time.Duration)(unsafe.Pointer(in.Interval))
	out.Retries = (*uint64)(unsafe.Pointer(in.Retries))
	return nil
}

// Convert_v1alpha3_HealthCheckConfig_To_v1beta2_HealthCheckConfig is an autogenerated conversion function.
func Convert_v1alpha3_HealthCheckConfig_To_v1beta2_HealthCheckConfig(in *HealthCheckConfig, out *v1beta2.HealthCheckConfig, s conversion.Scope) error {
	return autoConvert_v1alpha3_HealthCheckConfig_To_v1beta2_HealthCheckConfig(in, out, s)
}

func autoConvert_v1beta2_HealthCheckConfig_To_v1alpha3_HealthCheckConfig(in *v1beta2.HealthCheckConfig, out *HealthCheckConfig, s conversion.Scope) error {
	out.Test = *(*[]string)(unsafe.Pointer(&in.Test))
	out.Timeout = (*time.Duration)(unsafe.Pointer(in.Timeout))
	out.Interval = (*time.Duration)(unsafe.Pointer(in.Interval))
	out.Retries = (*uint64)(unsafe.Pointer(in.Retries))
	return nil
}

// Convert_v1beta2_HealthCheckConfig_To_v1alpha3_HealthCheckConfig is an autogenerated conversion function.
func Convert_v1beta2_HealthCheckConfig_To_v1alpha3_HealthCheckConfig(in *v1beta2.HealthCheckConfig, out *HealthCheckConfig, s conversion.Scope) error {
	return autoConvert_v1beta2_HealthCheckConfig_To_v1alpha3_HealthCheckConfig(in, out, s)
}

func autoConvert_v1alpha3_Owner_To_v1beta2_Owner(in *Owner, out *v1beta2.Owner, s conversion.Scope) error {
	out.ObjectMeta = in.ObjectMeta
	out.Owner = in.Owner
	return nil
}

// Convert_v1alpha3_Owner_To_v1beta2_Owner is an autogenerated conversion function.
func Convert_v1alpha3_Owner_To_v1beta2_Owner(in *Owner, out *v1beta2.Owner, s conversion.Scope) error {
	return autoConvert_v1alpha3_Owner_To_v1beta2_Owner(in, out, s)
}

func autoConvert_v1beta2_Owner_To_v1alpha3_Owner(in *v1beta2.Owner, out *Owner, s conversion.Scope) error {
	out.ObjectMeta = in.ObjectMeta
	out.Owner = in.Owner
	return nil
}

// Convert_v1beta2_Owner_To_v1alpha3_Owner is an autogenerated conversion function.
func Convert_v1beta2_Owner_To_v1alpha3_Owner(in *v1beta2.Owner, out *Owner, s conversion.Scope) error {
	return autoConvert_v1beta2_Owner_To_v1alpha3_Owner(in, out, s)
}

func autoConvert_v1alpha3_Placement_To_v1beta2_Placement(in *Placement, out *v1beta2.Placement, s conversion.Scope) error {
	out.Constraints = (*v1beta2.Constraints)(unsafe.Pointer(in.Constraints))
	return nil
}

// Convert_v1alpha3_Placement_To_v1beta2_Placement is an autogenerated conversion function.
func Convert_v1alpha3_Placement_To_v1beta2_Placement(in *Placement, out *v1beta2.Placement, s conversion.Scope) error {
	return autoConvert_v1alpha3_Placement_To_v1beta2_Placement(in, out, s)
}

func autoConvert_v1beta2_Placement_To_v1alpha3_Placement(in *v1beta2.Placement, out *Placement, s conversion.Scope) error {
	out.Constraints = (*Constraints)(unsafe.Pointer(in.Constraints))
	return nil
}

// Convert_v1beta2_Placement_To_v1alpha3_Placement is an autogenerated conversion function.
func Convert_v1beta2_Placement_To_v1alpha3_Placement(in *v1beta2.Placement, out *Placement, s conversion.Scope) error {
	return autoConvert_v1beta2_Placement_To_v1alpha3_Placement(in, out, s)
}

func autoConvert_v1alpha3_Resource_To_v1beta2_Resource(in *Resource, out *v1beta2.Resource, s conversion.Scope) error {
	out.NanoCPUs = in.NanoCPUs
	out.MemoryBytes = in.MemoryBytes
	return nil
}

// Convert_v1alpha3_Resource_To_v1beta2_Resource is an autogenerated conversion function.
func Convert_v1alpha3_Resource_To_v1beta2_Resource(in *Resource, out *v1beta2.Resource, s conversion.Scope) error {
	return autoConvert_v1alpha3_Resource_To_v1beta2_Resource(in, out, s)
}

func autoConvert_v1beta2_Resource_To_v1alpha3_Resource(in *v1beta2.Resource, out *Resource, s conversion.Scope) error {
	out.NanoCPUs = in.NanoCPUs
	out.MemoryBytes = in.MemoryBytes
	return nil
}

// Convert_v1beta2_Resource_To_v1alpha3_Resource is an autogenerated conversion function.
func Convert_v1beta2_Resource_To_v1alpha3_Resource(in *v1beta2.Resource, out *Resource, s conversion.Scope) error {
	return autoConvert_v1beta2_Resource_To_v1alpha3_Resource(in, out, s)
}

func autoConvert_v1alpha3_Resources_To_v1beta2_Resources(in *Resources, out *v1beta2.Resources, s conversion.Scope) error {
	out.Limits = (*v1beta2.Resource)(unsafe.Pointer(in.Limits))
	out.Reservations = (*v1beta2.Resource)(unsafe.Pointer(in.Reservations))
	return nil
}

// Convert_v1alpha3_Resources_To_v1beta2_Resources is an autogenerated conversion function.
func Convert_v1alpha3_Resources_To_v1beta2_Resources(in *Resources, out *v1beta2.Resources, s conversion.Scope) error {
	return autoConvert_v1alpha3_Resources_To_v1beta2_Resources(in, out, s)
}

func autoConvert_v1beta2_Resources_To_v1alpha3_Resources(in *v1beta2.Resources, out *Resources, s conversion.Scope) error {
	out.Limits = (*Resource)(unsafe.Pointer(in.Limits))
	out.Reservations = (*Resource)(unsafe.Pointer(in.Reservations))
	return nil
}

// Convert_v1beta2_Resources_To_v1alpha3_Resources is an autogenerated conversion function.
func Convert_v1beta2_Resources_To_v1alpha3_Resources(in *v1beta2.Resources, out *Resources, s conversion.Scope) error {
	return autoConvert_v1beta2_Resources_To_v1alpha3_Resources(in, out, s)
}

func autoConvert_v1alpha3_RestartPolicy_To_v1beta2_RestartPolicy(in *RestartPolicy, out *v1beta2.RestartPolicy, s conversion.Scope) error {
	out.Condition = in.Condition
	return nil
}

// Convert_v1alpha3_RestartPolicy_To_v1beta2_RestartPolicy is an autogenerated conversion function.
func Convert_v1alpha3_RestartPolicy_To_v1beta2_RestartPolicy(in *RestartPolicy, out *v1beta2.RestartPolicy, s conversion.Scope) error {
	return autoConvert_v1alpha3_RestartPolicy_To_v1beta2_RestartPolicy(in, out, s)
}

func autoConvert_v1beta2_RestartPolicy_To_v1alpha3_RestartPolicy(in *v1beta2.RestartPolicy, out *RestartPolicy, s conversion.Scope) error {
	out.Condition = in.Condition
	return nil
}

// Convert_v1beta2_RestartPolicy_To_v1alpha3_RestartPolicy is an autogenerated conversion function.
func Convert_v1beta2_RestartPolicy_To_v1alpha3_RestartPolicy(in *v1beta2.RestartPolicy, out *RestartPolicy, s conversion.Scope) error {
	return autoConvert_v1beta2_RestartPolicy_To_v1alpha3_RestartPolicy(in, out, s)
}

func autoConvert_v1alpha3_Scale_To_v1beta2_Scale(in *Scale, out *v1beta2.Scale, s conversion.Scope) error {
	out.ObjectMeta = in.ObjectMeta
	out.Spec = *(*map[string]int)(unsafe.Pointer(&in.Spec))
	out.Status = *(*map[string]int)(unsafe.Pointer(&in.Status))
	return nil
}

// Convert_v1alpha3_Scale_To_v1beta2_Scale is an autogenerated conversion function.
func Convert_v1alpha3_Scale_To_v1beta2_Scale(in *Scale, out *v1beta2.Scale, s conversion.Scope) error {
	return autoConvert_v1alpha3_Scale_To_v1beta2_Scale(in, out, s)
}

func autoConvert_v1beta2_Scale_To_v1alpha3_Scale(in *v1beta2.Scale, out *Scale, s conversion.Scope) error {
	out.ObjectMeta = in.ObjectMeta
	out.Spec = *(*map[string]int)(unsafe.Pointer(&in.Spec))
	out.Status = *(*map[string]int)(unsafe.Pointer(&in.Status))
	return nil
}

// Convert_v1beta2_Scale_To_v1alpha3_Scale is an autogenerated conversion function.
func Convert_v1beta2_Scale_To_v1alpha3_Scale(in *v1beta2.Scale, out *Scale, s conversion.Scope) error {
	return autoConvert_v1beta2_Scale_To_v1alpha3_Scale(in, out, s)
}

func autoConvert_v1alpha3_SecretConfig_To_v1beta2_SecretConfig(in *SecretConfig, out *v1beta2.SecretConfig, s conversion.Scope) error {
	out.Name = in.Name
	out.File = in.File
	if err := Convert_v1alpha3_External_To_v1beta2_External(&in.External, &out.External, s); err != nil {
		return err
	}
	out.Labels = *(*map[string]string)(unsafe.Pointer(&in.Labels))
	return nil
}

// Convert_v1alpha3_SecretConfig_To_v1beta2_SecretConfig is an autogenerated conversion function.
func Convert_v1alpha3_SecretConfig_To_v1beta2_SecretConfig(in *SecretConfig, out *v1beta2.SecretConfig, s conversion.Scope) error {
	return autoConvert_v1alpha3_SecretConfig_To_v1beta2_SecretConfig(in, out, s)
}

func autoConvert_v1beta2_SecretConfig_To_v1alpha3_SecretConfig(in *v1beta2.SecretConfig, out *SecretConfig, s conversion.Scope) error {
	out.Name = in.Name
	out.File = in.File
	if err := Convert_v1beta2_External_To_v1alpha3_External(&in.External, &out.External, s); err != nil {
		return err
	}
	out.Labels = *(*map[string]string)(unsafe.Pointer(&in.Labels))
	return nil
}

// Convert_v1beta2_SecretConfig_To_v1alpha3_SecretConfig is an autogenerated conversion function.
func Convert_v1beta2_SecretConfig_To_v1alpha3_SecretConfig(in *v1beta2.SecretConfig, out *SecretConfig, s conversion.Scope) error {
	return autoConvert_v1beta2_SecretConfig_To_v1alpha3_SecretConfig(in, out, s)
}

func autoConvert_v1alpha3_ServiceConfig_To_v1beta2_ServiceConfig(in *ServiceConfig, out *v1beta2.ServiceConfig, s conversion.Scope) error {
	out.Name = in.Name
	out.CapAdd = *(*[]string)(unsafe.Pointer(&in.CapAdd))
	out.CapDrop = *(*[]string)(unsafe.Pointer(&in.CapDrop))
	out.Command = *(*[]string)(unsafe.Pointer(&in.Command))
	out.Configs = *(*[]v1beta2.ServiceConfigObjConfig)(unsafe.Pointer(&in.Configs))
	if err := Convert_v1alpha3_DeployConfig_To_v1beta2_DeployConfig(&in.Deploy, &out.Deploy, s); err != nil {
		return err
	}
	out.Entrypoint = *(*[]string)(unsafe.Pointer(&in.Entrypoint))
	out.Environment = *(*map[string]*string)(unsafe.Pointer(&in.Environment))
	out.ExtraHosts = *(*[]string)(unsafe.Pointer(&in.ExtraHosts))
	out.Hostname = in.Hostname
	out.HealthCheck = (*v1beta2.HealthCheckConfig)(unsafe.Pointer(in.HealthCheck))
	out.Image = in.Image
	out.Ipc = in.Ipc
	out.Labels = *(*map[string]string)(unsafe.Pointer(&in.Labels))
	out.Pid = in.Pid
	out.Ports = *(*[]v1beta2.ServicePortConfig)(unsafe.Pointer(&in.Ports))
	out.Privileged = in.Privileged
	out.ReadOnly = in.ReadOnly
	out.Secrets = *(*[]v1beta2.ServiceSecretConfig)(unsafe.Pointer(&in.Secrets))
	out.StdinOpen = in.StdinOpen
	out.StopGracePeriod = (*time.Duration)(unsafe.Pointer(in.StopGracePeriod))
	out.Tmpfs = *(*[]string)(unsafe.Pointer(&in.Tmpfs))
	out.Tty = in.Tty
	out.User = (*int64)(unsafe.Pointer(in.User))
	out.Volumes = *(*[]v1beta2.ServiceVolumeConfig)(unsafe.Pointer(&in.Volumes))
	out.WorkingDir = in.WorkingDir
	// WARNING: in.PullSecret requires manual conversion: does not exist in peer-type
	// WARNING: in.PullPolicy requires manual conversion: does not exist in peer-type
	// WARNING: in.InternalPorts requires manual conversion: does not exist in peer-type
	// WARNING: in.InternalServiceType requires manual conversion: does not exist in peer-type
	return nil
}

func autoConvert_v1beta2_ServiceConfig_To_v1alpha3_ServiceConfig(in *v1beta2.ServiceConfig, out *ServiceConfig, s conversion.Scope) error {
	out.Name = in.Name
	out.CapAdd = *(*[]string)(unsafe.Pointer(&in.CapAdd))
	out.CapDrop = *(*[]string)(unsafe.Pointer(&in.CapDrop))
	out.Command = *(*[]string)(unsafe.Pointer(&in.Command))
	out.Configs = *(*[]ServiceConfigObjConfig)(unsafe.Pointer(&in.Configs))
	if err := Convert_v1beta2_DeployConfig_To_v1alpha3_DeployConfig(&in.Deploy, &out.Deploy, s); err != nil {
		return err
	}
	out.Entrypoint = *(*[]string)(unsafe.Pointer(&in.Entrypoint))
	out.Environment = *(*map[string]*string)(unsafe.Pointer(&in.Environment))
	out.ExtraHosts = *(*[]string)(unsafe.Pointer(&in.ExtraHosts))
	out.Hostname = in.Hostname
	out.HealthCheck = (*HealthCheckConfig)(unsafe.Pointer(in.HealthCheck))
	out.Image = in.Image
	out.Ipc = in.Ipc
	out.Labels = *(*map[string]string)(unsafe.Pointer(&in.Labels))
	out.Pid = in.Pid
	out.Ports = *(*[]ServicePortConfig)(unsafe.Pointer(&in.Ports))
	out.Privileged = in.Privileged
	out.ReadOnly = in.ReadOnly
	out.Secrets = *(*[]ServiceSecretConfig)(unsafe.Pointer(&in.Secrets))
	out.StdinOpen = in.StdinOpen
	out.StopGracePeriod = (*time.Duration)(unsafe.Pointer(in.StopGracePeriod))
	out.Tmpfs = *(*[]string)(unsafe.Pointer(&in.Tmpfs))
	out.Tty = in.Tty
	out.User = (*int64)(unsafe.Pointer(in.User))
	out.Volumes = *(*[]ServiceVolumeConfig)(unsafe.Pointer(&in.Volumes))
	out.WorkingDir = in.WorkingDir
	return nil
}

// Convert_v1beta2_ServiceConfig_To_v1alpha3_ServiceConfig is an autogenerated conversion function.
func Convert_v1beta2_ServiceConfig_To_v1alpha3_ServiceConfig(in *v1beta2.ServiceConfig, out *ServiceConfig, s conversion.Scope) error {
	return autoConvert_v1beta2_ServiceConfig_To_v1alpha3_ServiceConfig(in, out, s)
}

func autoConvert_v1alpha3_ServiceConfigObjConfig_To_v1beta2_ServiceConfigObjConfig(in *ServiceConfigObjConfig, out *v1beta2.ServiceConfigObjConfig, s conversion.Scope) error {
	out.Source = in.Source
	out.Target = in.Target
	out.UID = in.UID
	out.GID = in.GID
	out.Mode = (*uint32)(unsafe.Pointer(in.Mode))
	return nil
}

// Convert_v1alpha3_ServiceConfigObjConfig_To_v1beta2_ServiceConfigObjConfig is an autogenerated conversion function.
func Convert_v1alpha3_ServiceConfigObjConfig_To_v1beta2_ServiceConfigObjConfig(in *ServiceConfigObjConfig, out *v1beta2.ServiceConfigObjConfig, s conversion.Scope) error {
	return autoConvert_v1alpha3_ServiceConfigObjConfig_To_v1beta2_ServiceConfigObjConfig(in, out, s)
}

func autoConvert_v1beta2_ServiceConfigObjConfig_To_v1alpha3_ServiceConfigObjConfig(in *v1beta2.ServiceConfigObjConfig, out *ServiceConfigObjConfig, s conversion.Scope) error {
	out.Source = in.Source
	out.Target = in.Target
	out.UID = in.UID
	out.GID = in.GID
	out.Mode = (*uint32)(unsafe.Pointer(in.Mode))
	return nil
}

// Convert_v1beta2_ServiceConfigObjConfig_To_v1alpha3_ServiceConfigObjConfig is an autogenerated conversion function.
func Convert_v1beta2_ServiceConfigObjConfig_To_v1alpha3_ServiceConfigObjConfig(in *v1beta2.ServiceConfigObjConfig, out *ServiceConfigObjConfig, s conversion.Scope) error {
	return autoConvert_v1beta2_ServiceConfigObjConfig_To_v1alpha3_ServiceConfigObjConfig(in, out, s)
}

func autoConvert_v1alpha3_ServicePortConfig_To_v1beta2_ServicePortConfig(in *ServicePortConfig, out *v1beta2.ServicePortConfig, s conversion.Scope) error {
	out.Mode = in.Mode
	out.Target = in.Target
	out.Published = in.Published
	out.Protocol = in.Protocol
	return nil
}

// Convert_v1alpha3_ServicePortConfig_To_v1beta2_ServicePortConfig is an autogenerated conversion function.
func Convert_v1alpha3_ServicePortConfig_To_v1beta2_ServicePortConfig(in *ServicePortConfig, out *v1beta2.ServicePortConfig, s conversion.Scope) error {
	return autoConvert_v1alpha3_ServicePortConfig_To_v1beta2_ServicePortConfig(in, out, s)
}

func autoConvert_v1beta2_ServicePortConfig_To_v1alpha3_ServicePortConfig(in *v1beta2.ServicePortConfig, out *ServicePortConfig, s conversion.Scope) error {
	out.Mode = in.Mode
	out.Target = in.Target
	out.Published = in.Published
	out.Protocol = in.Protocol
	return nil
}

// Convert_v1beta2_ServicePortConfig_To_v1alpha3_ServicePortConfig is an autogenerated conversion function.
func Convert_v1beta2_ServicePortConfig_To_v1alpha3_ServicePortConfig(in *v1beta2.ServicePortConfig, out *ServicePortConfig, s conversion.Scope) error {
	return autoConvert_v1beta2_ServicePortConfig_To_v1alpha3_ServicePortConfig(in, out, s)
}

func autoConvert_v1alpha3_ServiceSecretConfig_To_v1beta2_ServiceSecretConfig(in *ServiceSecretConfig, out *v1beta2.ServiceSecretConfig, s conversion.Scope) error {
	out.Source = in.Source
	out.Target = in.Target
	out.UID = in.UID
	out.GID = in.GID
	out.Mode = (*uint32)(unsafe.Pointer(in.Mode))
	return nil
}

// Convert_v1alpha3_ServiceSecretConfig_To_v1beta2_ServiceSecretConfig is an autogenerated conversion function.
func Convert_v1alpha3_ServiceSecretConfig_To_v1beta2_ServiceSecretConfig(in *ServiceSecretConfig, out *v1beta2.ServiceSecretConfig, s conversion.Scope) error {
	return autoConvert_v1alpha3_ServiceSecretConfig_To_v1beta2_ServiceSecretConfig(in, out, s)
}

func autoConvert_v1beta2_ServiceSecretConfig_To_v1alpha3_ServiceSecretConfig(in *v1beta2.ServiceSecretConfig, out *ServiceSecretConfig, s conversion.Scope) error {
	out.Source = in.Source
	out.Target = in.Target
	out.UID = in.UID
	out.GID = in.GID
	out.Mode = (*uint32)(unsafe.Pointer(in.Mode))
	return nil
}

// Convert_v1beta2_ServiceSecretConfig_To_v1alpha3_ServiceSecretConfig is an autogenerated conversion function.
func Convert_v1beta2_ServiceSecretConfig_To_v1alpha3_ServiceSecretConfig(in *v1beta2.ServiceSecretConfig, out *ServiceSecretConfig, s conversion.Scope) error {
	return autoConvert_v1beta2_ServiceSecretConfig_To_v1alpha3_ServiceSecretConfig(in, out, s)
}

func autoConvert_v1alpha3_ServiceVolumeConfig_To_v1beta2_ServiceVolumeConfig(in *ServiceVolumeConfig, out *v1beta2.ServiceVolumeConfig, s conversion.Scope) error {
	out.Type = in.Type
	out.Source = in.Source
	out.Target = in.Target
	out.ReadOnly = in.ReadOnly
	return nil
}

// Convert_v1alpha3_ServiceVolumeConfig_To_v1beta2_ServiceVolumeConfig is an autogenerated conversion function.
func Convert_v1alpha3_ServiceVolumeConfig_To_v1beta2_ServiceVolumeConfig(in *ServiceVolumeConfig, out *v1beta2.ServiceVolumeConfig, s conversion.Scope) error {
	return autoConvert_v1alpha3_ServiceVolumeConfig_To_v1beta2_ServiceVolumeConfig(in, out, s)
}

func autoConvert_v1beta2_ServiceVolumeConfig_To_v1alpha3_ServiceVolumeConfig(in *v1beta2.ServiceVolumeConfig, out *ServiceVolumeConfig, s conversion.Scope) error {
	out.Type = in.Type
	out.Source = in.Source
	out.Target = in.Target
	out.ReadOnly = in.ReadOnly
	return nil
}

// Convert_v1beta2_ServiceVolumeConfig_To_v1alpha3_ServiceVolumeConfig is an autogenerated conversion function.
func Convert_v1beta2_ServiceVolumeConfig_To_v1alpha3_ServiceVolumeConfig(in *v1beta2.ServiceVolumeConfig, out *ServiceVolumeConfig, s conversion.Scope) error {
	return autoConvert_v1beta2_ServiceVolumeConfig_To_v1alpha3_ServiceVolumeConfig(in, out, s)
}

func autoConvert_v1alpha3_Stack_To_v1beta2_Stack(in *Stack, out *v1beta2.Stack, s conversion.Scope) error {
	out.ObjectMeta = in.ObjectMeta
	if in.Spec != nil {
		in, out := &in.Spec, &out.Spec
		*out = new(v1beta2.StackSpec)
		if err := Convert_v1alpha3_StackSpec_To_v1beta2_StackSpec(*in, *out, s); err != nil {
			return err
		}
	} else {
		out.Spec = nil
	}
	out.Status = (*v1beta2.StackStatus)(unsafe.Pointer(in.Status))
	return nil
}

// Convert_v1alpha3_Stack_To_v1beta2_Stack is an autogenerated conversion function.
func Convert_v1alpha3_Stack_To_v1beta2_Stack(in *Stack, out *v1beta2.Stack, s conversion.Scope) error {
	return autoConvert_v1alpha3_Stack_To_v1beta2_Stack(in, out, s)
}

func autoConvert_v1beta2_Stack_To_v1alpha3_Stack(in *v1beta2.Stack, out *Stack, s conversion.Scope) error {
	out.ObjectMeta = in.ObjectMeta
	if in.Spec != nil {
		in, out := &in.Spec, &out.Spec
		*out = new(StackSpec)
		if err := Convert_v1beta2_StackSpec_To_v1alpha3_StackSpec(*in, *out, s); err != nil {
			return err
		}
	} else {
		out.Spec = nil
	}
	out.Status = (*StackStatus)(unsafe.Pointer(in.Status))
	return nil
}

// Convert_v1beta2_Stack_To_v1alpha3_Stack is an autogenerated conversion function.
func Convert_v1beta2_Stack_To_v1alpha3_Stack(in *v1beta2.Stack, out *Stack, s conversion.Scope) error {
	return autoConvert_v1beta2_Stack_To_v1alpha3_Stack(in, out, s)
}

func autoConvert_v1alpha3_StackList_To_v1beta2_StackList(in *StackList, out *v1beta2.StackList, s conversion.Scope) error {
	out.ListMeta = in.ListMeta
	if in.Items != nil {
		in, out := &in.Items, &out.Items
		*out = make([]v1beta2.Stack, len(*in))
		for i := range *in {
			if err := Convert_v1alpha3_Stack_To_v1beta2_Stack(&(*in)[i], &(*out)[i], s); err != nil {
				return err
			}
		}
	} else {
		out.Items = nil
	}
	return nil
}

// Convert_v1alpha3_StackList_To_v1beta2_StackList is an autogenerated conversion function.
func Convert_v1alpha3_StackList_To_v1beta2_StackList(in *StackList, out *v1beta2.StackList, s conversion.Scope) error {
	return autoConvert_v1alpha3_StackList_To_v1beta2_StackList(in, out, s)
}

func autoConvert_v1beta2_StackList_To_v1alpha3_StackList(in *v1beta2.StackList, out *StackList, s conversion.Scope) error {
	out.ListMeta = in.ListMeta
	if in.Items != nil {
		in, out := &in.Items, &out.Items
		*out = make([]Stack, len(*in))
		for i := range *in {
			if err := Convert_v1beta2_Stack_To_v1alpha3_Stack(&(*in)[i], &(*out)[i], s); err != nil {
				return err
			}
		}
	} else {
		out.Items = nil
	}
	return nil
}

// Convert_v1beta2_StackList_To_v1alpha3_StackList is an autogenerated conversion function.
func Convert_v1beta2_StackList_To_v1alpha3_StackList(in *v1beta2.StackList, out *StackList, s conversion.Scope) error {
	return autoConvert_v1beta2_StackList_To_v1alpha3_StackList(in, out, s)
}

func autoConvert_v1alpha3_StackSpec_To_v1beta2_StackSpec(in *StackSpec, out *v1beta2.StackSpec, s conversion.Scope) error {
	if in.Services != nil {
		in, out := &in.Services, &out.Services
		*out = make([]v1beta2.ServiceConfig, len(*in))
		for i := range *in {
			if err := Convert_v1alpha3_ServiceConfig_To_v1beta2_ServiceConfig(&(*in)[i], &(*out)[i], s); err != nil {
				return err
			}
		}
	} else {
		out.Services = nil
	}
	out.Secrets = *(*map[string]v1beta2.SecretConfig)(unsafe.Pointer(&in.Secrets))
	out.Configs = *(*map[string]v1beta2.ConfigObjConfig)(unsafe.Pointer(&in.Configs))
	return nil
}

// Convert_v1alpha3_StackSpec_To_v1beta2_StackSpec is an autogenerated conversion function.
func Convert_v1alpha3_StackSpec_To_v1beta2_StackSpec(in *StackSpec, out *v1beta2.StackSpec, s conversion.Scope) error {
	return autoConvert_v1alpha3_StackSpec_To_v1beta2_StackSpec(in, out, s)
}

func autoConvert_v1beta2_StackSpec_To_v1alpha3_StackSpec(in *v1beta2.StackSpec, out *StackSpec, s conversion.Scope) error {
	if in.Services != nil {
		in, out := &in.Services, &out.Services
		*out = make([]ServiceConfig, len(*in))
		for i := range *in {
			if err := Convert_v1beta2_ServiceConfig_To_v1alpha3_ServiceConfig(&(*in)[i], &(*out)[i], s); err != nil {
				return err
			}
		}
	} else {
		out.Services = nil
	}
	out.Secrets = *(*map[string]SecretConfig)(unsafe.Pointer(&in.Secrets))
	out.Configs = *(*map[string]ConfigObjConfig)(unsafe.Pointer(&in.Configs))
	return nil
}

// Convert_v1beta2_StackSpec_To_v1alpha3_StackSpec is an autogenerated conversion function.
func Convert_v1beta2_StackSpec_To_v1alpha3_StackSpec(in *v1beta2.StackSpec, out *StackSpec, s conversion.Scope) error {
	return autoConvert_v1beta2_StackSpec_To_v1alpha3_StackSpec(in, out, s)
}

func autoConvert_v1alpha3_StackStatus_To_v1beta2_StackStatus(in *StackStatus, out *v1beta2.StackStatus, s conversion.Scope) error {
	out.Phase = v1beta2.StackPhase(in.Phase)
	out.Message = in.Message
	return nil
}

// Convert_v1alpha3_StackStatus_To_v1beta2_StackStatus is an autogenerated conversion function.
func Convert_v1alpha3_StackStatus_To_v1beta2_StackStatus(in *StackStatus, out *v1beta2.StackStatus, s conversion.Scope) error {
	return autoConvert_v1alpha3_StackStatus_To_v1beta2_StackStatus(in, out, s)
}

func autoConvert_v1beta2_StackStatus_To_v1alpha3_StackStatus(in *v1beta2.StackStatus, out *StackStatus, s conversion.Scope) error {
	out.Phase = StackPhase(in.Phase)
	out.Message = in.Message
	return nil
}

// Convert_v1beta2_StackStatus_To_v1alpha3_StackStatus is an autogenerated conversion function.
func Convert_v1beta2_StackStatus_To_v1alpha3_StackStatus(in *v1beta2.StackStatus, out *StackStatus, s conversion.Scope) error {
	return autoConvert_v1beta2_StackStatus_To_v1alpha3_StackStatus(in, out, s)
}

func autoConvert_v1alpha3_UpdateConfig_To_v1beta2_UpdateConfig(in *UpdateConfig, out *v1beta2.UpdateConfig, s conversion.Scope) error {
	out.Parallelism = (*uint64)(unsafe.Pointer(in.Parallelism))
	return nil
}

// Convert_v1alpha3_UpdateConfig_To_v1beta2_UpdateConfig is an autogenerated conversion function.
func Convert_v1alpha3_UpdateConfig_To_v1beta2_UpdateConfig(in *UpdateConfig, out *v1beta2.UpdateConfig, s conversion.Scope) error {
	return autoConvert_v1alpha3_UpdateConfig_To_v1beta2_UpdateConfig(in, out, s)
}

func autoConvert_v1beta2_UpdateConfig_To_v1alpha3_UpdateConfig(in *v1beta2.UpdateConfig, out *UpdateConfig, s conversion.Scope) error {
	out.Parallelism = (*uint64)(unsafe.Pointer(in.Parallelism))
	return nil
}

// Convert_v1beta2_UpdateConfig_To_v1alpha3_UpdateConfig is an autogenerated conversion function.
func Convert_v1beta2_UpdateConfig_To_v1alpha3_UpdateConfig(in *v1beta2.UpdateConfig, out *UpdateConfig, s conversion.Scope) error {
	return autoConvert_v1beta2_UpdateConfig_To_v1alpha3_UpdateConfig(in, out, s)
}
